@@ -48,7 +48,6 @@ from nuitka.Constants import (
     getConstantWeight,
     isMutable
 )
-from nuitka.PythonVersions import python_version
 
 from .BlobCodes import StreamData
 from .Emission import SourceCodeCollector
@@ -82,9 +81,14 @@ def generateConstantTrueReferenceCode(to_name, expression, emit, context):
 
     # No context or other knowledge needed, pylint: disable=unused-argument
 
-    emit(
-        "%s = Py_True;" % to_name
-    )
+    if to_name.c_type == "nuitka_bool":
+        emit(
+            "%s = NUITKA_BOOL_TRUE;" % to_name
+        )
+    else:
+        emit(
+            "%s = Py_True;" % to_name
+        )
 
 
 def generateConstantFalseReferenceCode(to_name, expression, emit, context):
@@ -92,9 +96,14 @@ def generateConstantFalseReferenceCode(to_name, expression, emit, context):
 
     # No context or other knowledge needed, pylint: disable=unused-argument
 
-    emit(
-        "%s = Py_False;" % to_name
-    )
+    if to_name.c_type == "nuitka_bool":
+        emit(
+            "%s = NUITKA_BOOL_FALSE;" % to_name
+        )
+    else:
+        emit(
+            "%s = Py_False;" % to_name
+        )
 
 
 def generateConstantEllipsisReferenceCode(to_name, expression, emit, context):
@@ -375,7 +384,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
     if constant_type is long:
         # See above, same for long values. Note: These are of course not
         # existent with Python3 which would have covered it before.
-        if constant_value >= 0 and constant_value <= max_unsigned_long:
+        if 0 <= constant_value <= max_unsigned_long:
             emit (
                 "%s = PyLong_FromUnsignedLong( %sul );" % (
                     constant_identifier,
@@ -384,7 +393,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
             )
 
             return
-        elif constant_value < 0 and constant_value >= min_signed_long:
+        elif 0 > constant_value >= min_signed_long:
             emit (
                 "%s = PyLong_FromLong( %sl );" % (
                     constant_identifier,
@@ -945,6 +954,9 @@ def getConstantAccess(to_name, constant, emit, context):
     # Many cases, because for each type, we may copy or optimize by creating
     # empty.  pylint: disable=too-many-branches,too-many-statements
 
+    if to_name.c_type == "nuitka_bool" and Options.isDebug():
+        assert False, constant
+
     if type(constant) is dict:
         if constant:
             for key, value in iterItems(constant):
@@ -1092,7 +1104,7 @@ def getConstantInitCodes(module_context):
 
 
 def allocateNestedConstants(module_context):
-    # Lots of types to deal with, pylint: disable=too-many-branches
+    # Lots of types to deal with.
 
     def considerForDeferral(constant_value):
         module_context.getConstantCode(constant_value)
@@ -1115,25 +1127,10 @@ def allocateNestedConstants(module_context):
             considerForDeferral(constant_value.stop)
         elif constant_type is xrange:
             if xrange is range:
-                if python_version >= 330:
-                    # For Python2 ranges, we use C long values directly.
-                    considerForDeferral(constant_value.start)
-                    considerForDeferral(constant_value.step)
-                    considerForDeferral(constant_value.stop)
-                else:
-                    parts = [
-                        int(value)
-                        for value in
-                        str(constant_value)[6:-1].split(',')
-                    ]
-
-                    if len(parts) <= 1:
-                        parts.append(0)
-                    if len(parts) <= 2:
-                        parts.append(1)
-
-                    for value in parts:
-                        considerForDeferral(value)
+                # For Python2 ranges, we use C long values directly.
+                considerForDeferral(constant_value.start)
+                considerForDeferral(constant_value.step)
+                considerForDeferral(constant_value.stop)
         elif constant_value in builtin_named_values_list:
             considerForDeferral(builtin_named_values[constant_value])
 
