@@ -19,18 +19,10 @@
 
 """
 
-template_frame_guard_cache_decl = """\
-static struct Nuitka_FrameObject *cache_%(frame_identifier)s = NULL;
-"""
-
-template_frame_guard_frame_decl = """\
-struct Nuitka_FrameObject *%(frame_identifier)s;
-"""
-
 # Frame in a function
 template_frame_guard_full_block = """\
-MAKE_OR_REUSE_FRAME( cache_%(frame_identifier)s, %(code_identifier)s, %(module_identifier)s, %(locals_size)s );
-%(frame_identifier)s = cache_%(frame_identifier)s;
+MAKE_OR_REUSE_FRAME( %(frame_cache_identifier)s, %(code_identifier)s, %(module_identifier)s, %(locals_size)s );
+%(frame_identifier)s = %(frame_cache_identifier)s;
 
 // Push the new frame as the currently active one.
 pushFrameStack( %(frame_identifier)s );
@@ -77,24 +69,24 @@ template_frame_guard_full_exception_handler = """\
 RESTORE_FRAME_EXCEPTION( %(frame_identifier)s );
 #endif
 
-if ( exception_tb == NULL )
+if ( %(exception_tb)s == NULL )
 {
-    exception_tb = %(tb_making)s;
+    %(exception_tb)s = %(tb_making)s;
 }
-else if ( exception_tb->tb_frame != &%(frame_identifier)s->m_frame )
+else if ( %(exception_tb)s->tb_frame != &%(frame_identifier)s->m_frame )
 {
-    exception_tb = ADD_TRACEBACK( exception_tb, %(frame_identifier)s, exception_lineno );
+    %(exception_tb)s = ADD_TRACEBACK( %(exception_tb)s, %(frame_identifier)s, %(exception_lineno)s );
 }
 
 // Attachs locals to frame if any.
 %(attach_locals)s
 
 // Release cached frame.
-if ( %(frame_identifier)s == cache_%(frame_identifier)s )
+if ( %(frame_identifier)s == %(frame_cache_identifier)s )
 {
     Py_DECREF( %(frame_identifier)s );
 }
-cache_%(frame_identifier)s = NULL;
+%(frame_cache_identifier)s = NULL;
 
 assertFrameObject( %(frame_identifier)s );
 
@@ -107,7 +99,7 @@ goto %(parent_exception_exit)s;
 
 # Frame for a module. TODO: Use it for functions called only once.
 # TODO: The once guard need not take a reference count in its frame class.
-template_frame_guard_once = """\
+template_frame_guard_once_block = """\
 // Frame without reuse.
 %(frame_identifier)s = MAKE_MODULE_FRAME( %(code_identifier)s, %(module_identifier)s );
 
@@ -128,18 +120,21 @@ popFrameStack();
 assertFrameObject( %(frame_identifier)s );
 
 goto %(no_exception_exit)s;
+"""
+
+template_frame_guard_once_exception_handler = """\
 %(frame_exception_exit)s:;
 #if %(needs_preserve)d
 RESTORE_FRAME_EXCEPTION( %(frame_identifier)s );
 #endif
 
-if ( exception_tb == NULL )
+if ( %(exception_tb)s == NULL )
 {
-    exception_tb = %(tb_making)s;
+    %(exception_tb)s = %(tb_making)s;
 }
 else if ( exception_tb->tb_frame != &%(frame_identifier)s->m_frame )
 {
-    exception_tb = ADD_TRACEBACK( exception_tb, %(frame_identifier)s, exception_lineno );
+    %(exception_tb)s = ADD_TRACEBACK( %(exception_tb)s, %(frame_identifier)s, %(exception_lineno)s );
 }
 
 // Put the previous frame back on top.
@@ -147,7 +142,7 @@ popFrameStack();
 
 // Return the error.
 goto %(parent_exception_exit)s;
-%(no_exception_exit)s:;"""
+"""
 
 # Frame in a generator, coroutine or asyncgen.
 template_frame_guard_generator = """\
@@ -175,16 +170,28 @@ Nuitka_Frame_MarkAsExecuting( %(context_identifier)s->m_frame );
 #if PYTHON_VERSION >= 300
 // Accept currently existing exception as the one to publish again when we
 // yield or yield from.
+{
+    PyThreadState *thread_state = PyThreadState_GET();
 
-PyThreadState *thread_state = PyThreadState_GET();
-
-%(context_identifier)s->m_frame->m_frame.f_exc_type = thread_state->exc_type;
+#if PYTHON_VERSION < 370
+    %(context_identifier)s->m_frame->m_frame.f_exc_type = EXC_TYPE( thread_state );
 if ( %(context_identifier)s->m_frame->m_frame.f_exc_type == Py_None ) %(context_identifier)s->m_frame->m_frame.f_exc_type = NULL;
-Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_type );
-%(context_identifier)s->m_frame->m_frame.f_exc_value = thread_state->exc_value;
-Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_value );
-%(context_identifier)s->m_frame->m_frame.f_exc_traceback = thread_state->exc_traceback;
-Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_traceback );
+    Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_type );
+%(context_identifier)s->m_frame->m_frame.f_exc_value = EXC_VALUE( thread_state );
+    Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_value );
+%(context_identifier)s->m_frame->m_frame.f_exc_traceback = EXC_TRACEBACK( thread_state );
+    Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_traceback );
+#else
+    %(context_identifier)s->m_exc_state.exc_type = EXC_TYPE( thread_state );
+    if ( %(context_identifier)s->m_exc_state.exc_type == Py_None ) %(context_identifier)s->m_exc_state.exc_type = NULL;
+    Py_XINCREF( %(context_identifier)s->m_exc_state.exc_type );
+    %(context_identifier)s->m_exc_state.exc_value = EXC_VALUE( thread_state );
+    Py_XINCREF( %(context_identifier)s->m_exc_state.exc_value );
+    %(context_identifier)s->m_exc_state.exc_traceback = EXC_TRACEBACK( thread_state );
+    Py_XINCREF( %(context_identifier)s->m_exc_state.exc_traceback );
+#endif
+}
+
 #endif
 
 // Framed code:
@@ -192,7 +199,11 @@ Py_XINCREF( %(context_identifier)s->m_frame->m_frame.f_exc_traceback );
 
 Nuitka_Frame_MarkAsNotExecuting( %(context_identifier)s->m_frame );
 
-#if PYTHON_VERSION >= 300
+#if PYTHON_VERSION >= 370
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_type );
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_value );
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_traceback );
+#elif PYTHON_VERSION >= 300
 Py_CLEAR( %(context_identifier)s->m_frame->m_frame.f_exc_type );
 Py_CLEAR( %(context_identifier)s->m_frame->m_frame.f_exc_value );
 Py_CLEAR( %(context_identifier)s->m_frame->m_frame.f_exc_traceback );
@@ -209,7 +220,11 @@ goto %(no_exception_exit)s;
 template_frame_guard_generator_return_handler = """\
 %(frame_return_exit)s:;
 
-#if PYTHON_VERSION >= 300
+#if PYTHON_VERSION >= 370
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_type );
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_value );
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_traceback );
+#elif PYTHON_VERSION >= 300
 Py_CLEAR( %(frame_identifier)s->m_frame.f_exc_type );
 Py_CLEAR( %(frame_identifier)s->m_frame.f_exc_value );
 Py_CLEAR( %(frame_identifier)s->m_frame.f_exc_traceback );
@@ -224,15 +239,15 @@ template_frame_guard_generator_exception_handler = """\
 %(frame_exception_exit)s:;
 
 // If it's not an exit exception, consider and create a traceback for it.
-if ( !EXCEPTION_MATCH_GENERATOR( exception_type ) )
+if ( !EXCEPTION_MATCH_GENERATOR( %(exception_type)s ) )
 {
-    if ( exception_tb == NULL )
+    if ( %(exception_tb)s == NULL )
     {
-        exception_tb = %(tb_making)s;
+        %(exception_tb)s = %(tb_making)s;
     }
-    else if ( exception_tb->tb_frame != &%(frame_identifier)s->m_frame )
+    else if ( %(exception_tb)s->tb_frame != &%(frame_identifier)s->m_frame )
     {
-        exception_tb = ADD_TRACEBACK( exception_tb, %(frame_identifier)s, exception_lineno );
+        %(exception_tb)s = ADD_TRACEBACK( %(exception_tb)s, %(frame_identifier)s, %(exception_lineno)s );
     }
 
 %(attach_locals)s
@@ -247,13 +262,18 @@ if ( !EXCEPTION_MATCH_GENERATOR( exception_type ) )
     assertFrameObject( %(frame_identifier)s );
 }
 
-#if PYTHON_VERSION >= 300
+#if PYTHON_VERSION >= 370
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_type );
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_value );
+Py_CLEAR( %(context_identifier)s->m_exc_state.exc_traceback );
+#elif PYTHON_VERSION >= 300
 Py_CLEAR( %(frame_identifier)s->m_frame.f_exc_type );
 Py_CLEAR( %(frame_identifier)s->m_frame.f_exc_value );
 Py_CLEAR( %(frame_identifier)s->m_frame.f_exc_traceback );
 #endif
 
 Py_DECREF( %(frame_identifier)s );
+
 // Return the error.
 goto %(parent_exception_exit)s;
 """
