@@ -139,7 +139,7 @@ class CollectionTracingMixin(object):
 
 
 class CollectionStartpointMixin(object):
-    # Many things are traces, pylint: disable=too-many-instance-attributes
+    # Many things are traced
 
     def __init__(self):
         # Variable assignments performed in here, last issued number, only used
@@ -157,8 +157,6 @@ class CollectionStartpointMixin(object):
 
         self.outline_functions = None
 
-        self.locals_scope = None
-
     def getLoopBreakCollections(self):
         return self.break_collections
 
@@ -312,15 +310,6 @@ class CollectionStartpointMixin(object):
     def updateVariablesFromCollection(self, old_collection):
         Variables.updateVariablesFromCollection(old_collection, self)
 
-    @contextlib.contextmanager
-    def makeLocalsDictContext(self, locals_scope):
-        old_locals_scope = self.locals_scope
-        self.locals_scope = locals_scope
-
-        yield
-
-        self.locals_scope = old_locals_scope
-
     @contextlib.contextmanager
     def makeAbortStackContext(self, catch_breaks, catch_continues,
                               catch_returns, catch_exceptions):
@@ -412,6 +401,9 @@ class TraceCollectionBase(CollectionTracingMixin):
             id(self)
         )
 
+    def getOwner(self):
+        return self.owner
+
     @staticmethod
     def signalChange(tags, source_ref, message):
         # This is monkey patched from another module.
@@ -514,7 +506,7 @@ class TraceCollectionBase(CollectionTracingMixin):
         return variable_trace
 
     def onLocalsUsage(self, locals_owner):
-        self.onLocalsDictEscaped(locals_owner.getLocalsScope())
+        self.onLocalsDictEscaped(locals_owner.getFunctionLocalsScope())
 
         result = []
 
@@ -548,7 +540,9 @@ class TraceCollectionBase(CollectionTracingMixin):
             return None
 
         assert expression.isExpression(), expression
-        assert expression.parent, expression
+
+        parent = expression.parent
+        assert parent, expression
 
         # Now compute this expression, allowing it to replace itself with
         # something else as part of a local peep hole optimization.
@@ -569,7 +563,7 @@ class TraceCollectionBase(CollectionTracingMixin):
             )
 
         if new_node is not expression:
-            expression.replaceWith(new_node)
+            parent.replaceChild(expression, new_node)
 
         return new_node
 
@@ -719,9 +713,6 @@ class TraceCollectionBase(CollectionTracingMixin):
             catch_exceptions = catch_exceptions
         )
 
-    def makeLocalsDictContext(self, locals_scope):
-        return self.parent.makeLocalsDictContext(locals_scope)
-
     def onLocalsDictEscaped(self, locals_scope):
         self.parent.onLocalsDictEscaped(locals_scope)
 
@@ -774,7 +765,7 @@ class TraceCollectionBranch(TraceCollectionBase):
             )
 
             if result is not branch:
-                branch.replaceWith(result)
+                branch.parent.replaceChild(branch, result)
         else:
             self.onExpression(
                 expression = branch
@@ -831,9 +822,14 @@ class TraceCollectionFunction(CollectionStartpointMixin,
             self.variable_actives[closure_variable] = 0
 
         # TODO: Have special function type for exec functions stuff.
-        if function_body.locals_scope is not None:
-            for locals_dict_variable in function_body.locals_scope.variables.values():
-                self._initVariableUninit(locals_dict_variable)
+        locals_scope = function_body.getFunctionLocalsScope()
+
+        if locals_scope is not None:
+            if not locals_scope.isMarkedForPropagation():
+                for locals_dict_variable in locals_scope.variables.values():
+                    self._initVariableUninit(locals_dict_variable)
+            else:
+                function_body.locals_scope = None
 
 
 class TraceCollectionModule(CollectionStartpointMixin,
