@@ -23,14 +23,20 @@
 
 from __future__ import print_function
 
+import hashlib
 import os
+import pickle
 import re
 import subprocess
 import sys
 import tempfile
 import time
 
-from nuitka.tools.testing.Common import addToPythonPath, withPythonPathChange
+from nuitka.tools.testing.Common import (
+    addToPythonPath,
+    getTestingCPythonOutputsCacheDir,
+    withPythonPathChange
+)
 from nuitka.tools.testing.OutputComparison import compareOutput
 from nuitka.Tracing import my_print
 
@@ -45,9 +51,6 @@ def displayOutput(stdout, stderr):
         my_print(stderr)
 
 
-
-
-
 def checkNoPermissionError(output):
     # Forms of permission errors.
     for candidate in (b"Permission denied:",
@@ -64,6 +67,66 @@ def checkNoPermissionError(output):
     return True
 
 
+def getCPythonResults(cpython_cmd, cpython_cached):
+    cached = False
+    if cpython_cached:
+        # TODO: Hashing stuff and creating cache filename is duplicate code
+        # and should be shared.
+
+        hash_input = " -- ".join(cpython_cmd)
+        if str is not bytes:
+            hash_input = hash_input.encode("utf8")
+
+        command_hash = hashlib.md5(hash_input)
+
+        cache_filename = os.path.join(
+            getTestingCPythonOutputsCacheDir(),
+            command_hash.hexdigest()
+        )
+
+        if os.path.exists(cache_filename):
+            with open(cache_filename, "rb") as cache_file:
+                cpython_time, stdout_cpython, stderr_cpython, exit_cpython = \
+                  pickle.load(cache_file)
+                cached = True
+
+    if not cached:
+        start_time = time.time()
+
+        # Try a coupile of times for permission denied, on Windows it can
+        # be transient.
+        for _i in range(5):
+            with withPythonPathChange(os.getcwd()):
+                process = subprocess.Popen(
+                    args   = cpython_cmd,
+                    stdout = subprocess.PIPE,
+                    stderr = subprocess.PIPE
+                )
+
+            stdout_cpython, stderr_cpython = process.communicate()
+            exit_cpython = process.returncode
+
+            if checkNoPermissionError(stdout_cpython) and \
+               checkNoPermissionError(stderr_cpython):
+                break
+
+            my_print("Retrying CPython due to permission problems after delay.")
+            time.sleep(2)
+
+            start_time = time.time()
+
+        cpython_time = time.time() - start_time
+
+        if cpython_cached:
+            with open(cache_filename, "wb") as cache_file:
+                pickle.dump(
+                    (cpython_time, stdout_cpython, stderr_cpython, exit_cpython),
+                    cache_file
+                )
+
+    return cpython_time, stdout_cpython, stderr_cpython, exit_cpython
+
+
 def main():
     # Of course many cases to deal with, pylint: disable=too-many-branches,too-many-locals,too-many-statements
     from nuitka.utils.Execution import check_output
@@ -103,7 +166,7 @@ def main():
     original_file      = hasArg("original_file")
     no_warnings        = not hasArg("warnings")
     full_compat        = not hasArg("improved")
-
+    cpython_cached     = hasArg("cpython_cache")
     syntax_errors      = hasArg("syntax_errors")
 
     plugins_enabled = []
@@ -308,16 +371,16 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         addToPythonPath(os.path.dirname(os.path.abspath(filename)))
 
     if keep_python_path or binary_python_path:
-        extra_options.append("--keep-pythonpath")
+        extra_options.append("--execute-with-pythonpath")
 
     if recurse_none:
-        extra_options.append("--recurse-none")
+        extra_options.append("--nofollow-imports")
 
     if recurse_all:
-        extra_options.append("--recurse-all")
+        extra_options.append("--follow-imports")
 
     if recurse_not:
-        extra_options.extend("--recurse-not-to=" + v for v in recurse_not)
+        extra_options.extend("--nofollow-import-to=" + v for v in recurse_not)
 
     if coverage_mode:
         extra_options.append("--must-not-re-execute")
@@ -333,13 +396,13 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
     if not two_step_execution:
         if module_mode:
             nuitka_cmd = nuitka_call + extra_options + \
-              ["--execute", "--module", filename]
+              ["--run", "--module", filename]
         elif standalone_mode:
             nuitka_cmd = nuitka_call + extra_options + \
-              ["--execute", "--standalone", filename]
+              ["--run", "--standalone", filename]
         else:
             nuitka_cmd = nuitka_call + extra_options + \
-              ["--execute", filename]
+              ["--run", filename]
 
         if no_site:
             nuitka_cmd.insert(len(nuitka_cmd) - 1, "--python-flag=-S")
@@ -382,7 +445,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
             exe_filename = exe_filename[:-3]
 
         exe_filename = exe_filename.replace(')', "").replace('(', "")
-        exe_filename += ".exe"
+        exe_filename += (".exe" if os.name == "nt" else ".bin")
 
         nuitka_cmd2 = [
             os.path.join(output_dir, exe_filename)
@@ -394,29 +457,10 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         my_print("CPython command:", *cpython_cmd)
 
     if comparison_mode:
-        start_time = time.time()
-
-        # Try a coupile of times for permission denied, on Windows it can
-        # be transient.
-        for _i in range(5):
-            with withPythonPathChange(os.getcwd()):
-                process = subprocess.Popen(
-                    args   = cpython_cmd,
-                    stdout = subprocess.PIPE,
-                    stderr = subprocess.PIPE
-                )
-
-            stdout_cpython, stderr_cpython = process.communicate()
-            exit_cpython = process.returncode
-
-            if checkNoPermissionError(stdout_cpython) and \
-               checkNoPermissionError(stderr_cpython):
-                break
-
-            my_print("Retrying CPython due to permission problems after delay.")
-            time.sleep(2)
-
-        cpython_time = time.time() - start_time
+        cpython_time, stdout_cpython, stderr_cpython, exit_cpython = getCPythonResults(
+            cpython_cmd    = cpython_cmd,
+            cpython_cached = cpython_cached
+        )
 
     if comparison_mode and not silent_mode:
         displayOutput(stdout_cpython, stderr_cpython)
